# Copyright WillianFuks
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""
Tests for summary.py module.
"""


import os

import pandas as pd
import pytest

import causalimpact.summary as summarizer


@pytest.fixture
def summary_data():
    data = [
        [5.123, 10.456],
        [4.234, 9.567],
        [3.123, 8.456],
        [6.234, 9.567],
        [3.123, 10.456],
        [2.234, 4.567],
        [6.123, 9.456],
        [0.2345, 0.5678],
        [0.1234, 0.4567],
        [0.2345, 0.5678]
    ]
    data = pd.DataFrame(
        data,
        columns=['average', 'cumulative'],
        index=[
            'actual',
            'predicted',
            'predicted_lower',
            'predicted_upper',
            'abs_effect',
            'abs_effect_lower',
            'abs_effect_upper',
            'rel_effect',
            'rel_effect_lower',
            'rel_effect_upper'
        ]
    )
    return data


def test_summary_raises(summary_data):
    with pytest.raises(ValueError):
        summarizer.summary(summary_data, 0.1, output='test')


def test_output_summary_single_digit(summary_data, fix_path):
    summarizer.summary_data = summary_data
    alpha = 0.1
    p_value = 0.459329

    result = summarizer.summary(summary_data, p_value, alpha, digits=1)
    expected = open(os.path.join(
                    fix_path, 'test_output_summary_single_digit')).read().strip()
    assert result == expected


def test_report_summary_single_digit(summary_data, fix_path):
    # detected positive signal but with no significance.
    alpha = 0.1
    p_value = 0.5
    summary_data['average']['rel_effect'] = 0.41
    summary_data['average']['rel_effect_lower'] = -0.30
    summary_data['average']['rel_effect_upper'] = 0.30

    result = summarizer.summary(summary_data, p_value, alpha=alpha, output='report',
                                digits=1)
    expected = open(os.path.join(
                    fix_path, 'test_report_summary_single_digit')).read().strip()
    assert result == expected


def test_output_summary_1(summary_data, fix_path):
    alpha = 0.1
    p_value = 0.459329

    result = summarizer.summary(summary_data, p_value, alpha=alpha)
    expected = open(os.path.join(fix_path, 'test_output_summary_1')).read().strip()
    assert result == expected


def test_report_summary_1(summary_data, fix_path):
    # detected positive signal but with no significance.
    alpha = 0.1
    p_value = 0.5
    summary_data['average']['rel_effect'] = 0.41
    summary_data['average']['rel_effect_lower'] = -0.30
    summary_data['average']['rel_effect_upper'] = 0.30

    result = summarizer.summary(summary_data, p_value, alpha=alpha, output='report')
    expected = open(os.path.join(fix_path, 'test_report_summary_1')).read().strip()
    assert result == expected


def test_report_summary_2(summary_data, fix_path):
    # detected positive signal with significance.
    alpha = 0.1
    p_value = 0.05
    summary_data['average']['rel_effect'] = 0.41
    summary_data['average']['rel_effect_lower'] = 0.434
    summary_data['average']['rel_effect_upper'] = 0.234

    result = summarizer.summary(summary_data, p_value, alpha=alpha, output='report')
    expected = open(os.path.join(fix_path, 'test_report_summary_2')).read().strip()
    assert result == expected


def test_report_summary_3(summary_data, fix_path):
    # detected negative signal but with no significance.
    summary_data['average']['rel_effect'] = -0.343
    summary_data['average']['rel_effect_lower'] = -0.434
    summary_data['average']['rel_effect_upper'] = 0.234
    alpha = 0.1
    p_value = 0.5

    result = summarizer.summary(summary_data, p_value, alpha=alpha, output='report')
    expected = open(os.path.join(fix_path, 'test_report_summary_3')).read().strip()
    assert result == expected


def test_report_summary_4(summary_data, fix_path):
    # detected negative signal with significance.
    summary_data['average']['rel_effect'] = -0.343
    summary_data['average']['rel_effect_lower'] = -0.434
    summary_data['average']['rel_effect_upper'] = -0.234
    alpha = 0.1
    p_value = 0.05

    result = summarizer.summary(summary_data, p_value, alpha=alpha, output='report')
    expected = open(os.path.join(fix_path, 'test_report_summary_4')).read().strip()
    assert result == expected



# Copyright WillianFuks
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import numpy as np
import pandas as pd
import pytest

from causalimpact.misc import get_z_score, standardize, unstandardize


def test_basic_standardize():
    data = {
        'c1': [1, 4, 8, 9, 10],
        'c2': [4, 8, 12, 16, 20]
    }
    data = pd.DataFrame(data)
    result, (mu, sig) = standardize(data)

    np.testing.assert_array_almost_equal(
        np.zeros(data.shape[1]),
        result.mean().values
    )

    np.testing.assert_array_almost_equal(
        np.ones(data.shape[1]),
        result.std(ddof=0).values
    )


def test_standardize_with_integer_column_names():
    # https://github.com/WillianFuks/tfcausalimpact/issues/17
    data = {
        'c1': [1, 4, 8, 9, 10],
        0: [4, 8, 12, 16, 20]
    }
    data = pd.DataFrame(data)
    result, (mu, sig) = standardize(data)

    np.testing.assert_array_almost_equal(
        np.zeros(data.shape[1]),
        result.mean().values
    )

    np.testing.assert_array_almost_equal(
        np.ones(data.shape[1]),
        result.std(ddof=0).values
    )


def test_standardize_w_various_distinct_inputs():
    test_data = [[1, 2, 1], [1, np.nan, 3], [10, 20, 30]]
    test_data = [pd.DataFrame(data, dtype="float") for data in test_data]
    for data in test_data:
        result, (mu, sig) = standardize(data)
        pd.testing.assert_frame_equal(unstandardize(result, (mu, sig)), data)


def test_standardize_raises_single_input():
    with pytest.raises(ValueError):
        standardize(pd.DataFrame([1]))


def test_get_z_score():
    assert get_z_score(0.5) == 0.
    assert round(float(get_z_score(0.9177)), 2) == 1.39



# Copyright WillianFuks
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""
Tests for module main.py. Fixtures comes from file conftest.py located at the same dir
of this file.
"""


import mock
import numpy as np
import pandas as pd
import pytest
import tensorflow as tf
import tensorflow_probability as tfp
from numpy.testing import assert_array_equal
from pandas.testing import assert_frame_equal

from causalimpact import CausalImpact
from causalimpact.misc import standardize

seed = 123
tf.random.set_seed(seed)
np.random.seed(seed)


def test_default_causal_cto(rand_data, pre_int_period, post_int_period):
    ci = CausalImpact(rand_data, pre_int_period, post_int_period)
    assert_frame_equal(ci.data, rand_data)
    assert ci.pre_period == pre_int_period
    assert ci.post_period == post_int_period
    new_rand_data = rand_data.set_index(pd.date_range('2020-01-01',
                                        periods=len(rand_data)))
    new_rand_data = tfp.sts.regularize_series(new_rand_data).astype(np.float32)
    pre_data = new_rand_data.iloc[pre_int_period[0]: pre_int_period[1] + 1, :]
    assert_frame_equal(ci.pre_data, pre_data)
    post_data = new_rand_data.iloc[post_int_period[0]: post_int_period[1] + 1, :]
    assert_frame_equal(ci.post_data, post_data)
    assert ci.alpha == 0.05
    normed_pre_data, (mu, sig) = standardize(pre_data)
    assert_frame_equal(ci.normed_pre_data, normed_pre_data)
    normed_post_data = (post_data - mu) / sig
    assert_frame_equal(ci.normed_post_data, normed_post_data)
    assert ci.mu_sig == (mu[0], sig[0])
    assert ci.model_args == {'fit_method': 'vi', 'niter': 1000, 'prior_level_sd': 0.01,
                             'season_duration': 1, 'nseasons': 1, 'standardize': True}
    assert isinstance(ci.model, tfp.sts.Sum)
    design_matrix = ci.model.components[1].design_matrix.to_dense()
    assert_array_equal(
        design_matrix,
        pd.concat([normed_pre_data, normed_post_data]).astype(np.float32).iloc[:, 1:]
    )
    assert ci.inferences is not None
    assert ci.inferences.index.dtype == rand_data.index.dtype
    assert ci.summary_data is not None
    assert ci.p_value > 0 and ci.p_value < 1
    assert ci.model_args['niter'] == 1000
    assert ci.model_samples is not None


@pytest.mark.slow
def test_default_causal_cto_with_date_index(date_rand_data, pre_str_period,
                                            post_str_period):
    ci = CausalImpact(date_rand_data, pre_str_period, post_str_period)
    assert_frame_equal(ci.data, date_rand_data)
    assert ci.pre_period == pre_str_period
    assert ci.post_period == post_str_period
    new_date_rand_data = tfp.sts.regularize_series(date_rand_data).astype(np.float32)
    pre_data = new_date_rand_data.loc[pre_str_period[0]: pre_str_period[1], :]
    assert_frame_equal(ci.pre_data, pre_data)
    post_data = new_date_rand_data.loc[post_str_period[0]: post_str_period[1], :]
    assert_frame_equal(ci.post_data, post_data)
    assert ci.alpha == 0.05
    normed_pre_data, (mu, sig) = standardize(pre_data)
    assert_frame_equal(ci.normed_pre_data, normed_pre_data)
    normed_post_data = (post_data - mu) / sig
    assert_frame_equal(ci.normed_post_data, normed_post_data)
    assert ci.mu_sig == (mu[0], sig[0])
    assert ci.model_args == {'fit_method': 'vi', 'niter': 1000, 'prior_level_sd': 0.01,
                             'season_duration': 1, 'nseasons': 1, 'standardize': True}
    assert isinstance(ci.model, tfp.sts.Sum)
    design_matrix = ci.model.components[1].design_matrix.to_dense()
    assert_array_equal(
        design_matrix,
        pd.concat([normed_pre_data, normed_post_data]).astype(np.float32).iloc[:, 1:]
    )
    assert ci.inferences is not None
    assert ci.inferences.index.dtype == date_rand_data.index.dtype
    assert ci.summary_data is not None
    assert ci.p_value > 0 and ci.p_value < 1
    assert ci.model_args['niter'] == 1000
    assert ci.model_samples is not None


@pytest.mark.slow
def test_default_causal_cto_no_covariates(rand_data, pre_int_period, post_int_period):
    rand_data = pd.DataFrame(rand_data.iloc[:, 0])
    ci = CausalImpact(rand_data, pre_int_period, post_int_period)
    assert_frame_equal(ci.data, rand_data)
    assert ci.pre_period == pre_int_period
    assert ci.post_period == post_int_period
    new_rand_data = rand_data.set_index(pd.date_range('2020-01-01',
                                        periods=len(rand_data)))
    new_rand_data = tfp.sts.regularize_series(new_rand_data).astype(np.float32)
    pre_data = new_rand_data.iloc[pre_int_period[0]: pre_int_period[1] + 1, :]
    assert_frame_equal(ci.pre_data, pre_data)
    post_data = new_rand_data.iloc[post_int_period[0]: post_int_period[1] + 1, :]
    assert_frame_equal(ci.post_data, post_data)
    assert ci.alpha == 0.05
    normed_pre_data, (mu, sig) = standardize(pre_data)
    assert_frame_equal(ci.normed_pre_data, normed_pre_data)
    normed_post_data = (post_data - mu) / sig
    assert_frame_equal(ci.normed_post_data, normed_post_data)
    assert ci.mu_sig == (mu[0], sig[0])
    assert ci.model_args == {'fit_method': 'vi', 'niter': 1000, 'prior_level_sd': 0.01,
                             'season_duration': 1, 'nseasons': 1, 'standardize': True}
    assert isinstance(ci.model, tfp.sts.Sum)
    assert ci.inferences is not None
    assert ci.inferences.index.dtype == rand_data.index.dtype
    assert ci.summary_data is not None
    assert ci.p_value > 0 and ci.p_value < 1
    assert ci.model_args['niter'] == 1000
    assert ci.model_samples is not None


def test_default_causal_cto_with_np_array(rand_data, pre_int_period, post_int_period):
    data = rand_data.values
    ci = CausalImpact(data, pre_int_period, post_int_period)
    assert_array_equal(ci.data, data)
    assert ci.pre_period == pre_int_period
    assert ci.post_period == post_int_period
    data = pd.DataFrame(data)
    new_data = data.set_index(pd.date_range('2020-01-01', periods=len(data)))
    new_data = tfp.sts.regularize_series(new_data).astype(np.float32)
    pre_data = new_data.iloc[pre_int_period[0]: pre_int_period[1] + 1, :]
    assert_frame_equal(ci.pre_data, pre_data)
    post_data = new_data.iloc[post_int_period[0]: post_int_period[1] + 1, :]
    assert_frame_equal(ci.post_data, post_data)
    assert ci.alpha == 0.05
    normed_pre_data, (mu, sig) = standardize(pre_data)
    assert_frame_equal(ci.normed_pre_data, normed_pre_data)
    normed_post_data = (post_data - mu) / sig
    assert_frame_equal(ci.normed_post_data, normed_post_data)
    assert ci.mu_sig == (mu[0], sig[0])
    assert ci.model_args == {'fit_method': 'vi', 'niter': 1000, 'prior_level_sd': 0.01,
                             'season_duration': 1, 'nseasons': 1, 'standardize': True}
    assert isinstance(ci.model, tfp.sts.Sum)
    design_matrix = ci.model.components[1].design_matrix.to_dense()
    assert_array_equal(
        design_matrix,
        pd.concat([normed_pre_data, normed_post_data]).astype(np.float32).iloc[:, 1:]
    )
    assert ci.inferences is not None
    assert ci.inferences.index.dtype == data.index.dtype
    assert ci.summary_data is not None
    assert ci.p_value > 0 and ci.p_value < 1
    assert ci.model_args['niter'] == 1000
    assert ci.model_samples is not None


@pytest.mark.slow
def test_causal_cto_with_no_standardization(rand_data, pre_int_period, post_int_period):
    ci = CausalImpact(rand_data, pre_int_period, post_int_period, model_args=dict(
        standardize=False, fit_method='vi'))
    assert ci.normed_pre_data is None
    assert ci.normed_post_data is None
    assert ci.mu_sig is None
    assert ci.p_value > 0 and ci.p_value < 1


@pytest.mark.slow
def test_causal_cto_with_seasons(date_rand_data, pre_str_period, post_str_period):
    ci = CausalImpact(date_rand_data, pre_str_period, post_str_period,
                      model_args={'nseasons': 7, 'season_duration': 2,
                                  'fit_method': 'vi'})
    assert len(ci.model.components) == 3
    seasonal_component = ci.model.components[2]
    assert seasonal_component.num_seasons == 7
    assert seasonal_component.num_steps_per_season == 2


def test_plotter(monkeypatch, rand_data, pre_int_period, post_int_period):
    plotter_mock = mock.Mock()
    fit_mock = mock.Mock()
    process_mock = mock.Mock()
    summarize_mock = mock.Mock()
    monkeypatch.setattr('causalimpact.main.CausalImpact._fit_model', fit_mock)
    monkeypatch.setattr('causalimpact.main.CausalImpact._summarize_inferences',
                        summarize_mock)
    monkeypatch.setattr('causalimpact.main.CausalImpact._process_posterior_inferences',
                        process_mock)
    monkeypatch.setattr('causalimpact.main.plotter', plotter_mock)
    ci = CausalImpact(rand_data, pre_int_period, post_int_period,
                      model_args={'fit_method': 'vi'})
    ci.inferences = 'inferences'
    ci.pre_data = 'pre_data'
    ci.post_data = 'post_data'
    ci._mask = slice(0, len(ci.post_data))
    ci.plot()
    plotter_mock.plot.assert_called_with('inferences', 'pre_data', 'post_data',
                                         panels=['original', 'pointwise', 'cumulative'],
                                         figsize=(10, 7), show=True)


def test_summarizer(monkeypatch, rand_data, pre_int_period, post_int_period):
    summarizer_mock = mock.Mock()
    fit_mock = mock.Mock()
    process_mock = mock.Mock()
    summarize_mock = mock.Mock()
    monkeypatch.setattr('causalimpact.main.CausalImpact._fit_model', fit_mock)
    monkeypatch.setattr('causalimpact.main.CausalImpact._summarize_inferences',
                        summarize_mock)
    monkeypatch.setattr('causalimpact.main.CausalImpact._process_posterior_inferences',
                        process_mock)
    monkeypatch.setattr('causalimpact.main.summarizer', summarizer_mock)
    ci = CausalImpact(rand_data, pre_int_period, post_int_period,
                      model_args={'fit_method': 'vi'})
    ci.summary_data = 'summary_data'
    ci.p_value = 0.5
    ci.alpha = 0.05
    ci.summary()
    summarizer_mock.summary.assert_called_with('summary_data', 0.5, 0.05, 'summary', 2)

    with pytest.raises(ValueError) as excinfo:
        ci.summary(digits='1')
    assert str(excinfo.value) == ('Input value for digits must be integer. Received '
                                  '"<class \'str\'>" instead.')


def test_causal_cto_with_custom_model_and_seasons(rand_data, pre_int_period,
                                                  post_int_period):
    pre_data = rand_data.loc[pre_int_period[0]: pre_int_period[1], :]
    observed_time_series = pre_data.iloc[:, 0].astype(np.float32)
    level = tfp.sts.LocalLevel(observed_time_series=observed_time_series)
    seasonal = tfp.sts.Seasonal(num_seasons=7, num_steps_per_season=2,
                                observed_time_series=observed_time_series)
    model = tfp.sts.Sum([level, seasonal], observed_time_series=observed_time_series)

    ci = CausalImpact(rand_data, pre_int_period, post_int_period, model=model,
                      model_args={'fit_method': 'vi'})

    assert len(ci.model.components) == 2
    assert isinstance(ci.model.components[0], tfp.sts.LocalLevel)
    assert isinstance(ci.model.components[1], tfp.sts.Seasonal)
    seasonal_component = ci.model.components[-1]
    assert seasonal_component.num_seasons == 7
    assert seasonal_component.num_steps_per_season == 2
    assert ci.inferences.index.dtype == rand_data.index.dtype


def test_default_causal_cto_vi_method(rand_data, pre_int_period, post_int_period):
    freq_rand_data = rand_data.set_index(
        pd.date_range(start='2020-01-01', periods=len(rand_data))
    ).astype(np.float32).asfreq(pd.offsets.DateOffset(days=1))
    ci = CausalImpact(rand_data, pre_int_period, post_int_period, model_args=dict(
        fit_method='vi'))
    assert_frame_equal(ci.data, rand_data)
    assert ci.pre_period == pre_int_period
    assert ci.post_period == post_int_period
    pre_data = freq_rand_data.iloc[pre_int_period[0]: pre_int_period[1] + 1, :]
    assert_frame_equal(ci.pre_data, pre_data)
    post_data = freq_rand_data.iloc[post_int_period[0]: post_int_period[1] + 1, :]
    assert_frame_equal(ci.post_data, post_data)
    assert ci.alpha == 0.05
    normed_pre_data, (mu, sig) = standardize(pre_data)
    assert_frame_equal(ci.normed_pre_data, normed_pre_data)
    normed_post_data = (post_data - mu) / sig
    assert_frame_equal(ci.normed_post_data, normed_post_data)
    assert ci.mu_sig == (mu[0], sig[0])
    assert ci.model_args == {'fit_method': 'vi', 'niter': 1000, 'prior_level_sd': 0.01,
                             'season_duration': 1, 'nseasons': 1, 'standardize': True}
    assert isinstance(ci.model, tfp.sts.Sum)
    design_matrix = ci.model.components[1].design_matrix.to_dense()
    assert_array_equal(
        design_matrix,
        pd.concat([normed_pre_data, normed_post_data]).astype(np.float32).iloc[:, 1:]
    )
    assert ci.inferences is not None
    assert ci.inferences.index.dtype == rand_data.index.dtype
    assert ci.summary_data is not None
    assert ci.p_value > 0 and ci.p_value < 1
    assert ci.model_args['niter'] == 1000
    assert ci.model_samples is not None
    assert ci.model_kernel_results is None


def test_default_model_arma_data():
    data = pd.read_csv('tests/fixtures/arma_data.csv')
    data.iloc[70:, 0] += 5

    pre_period = [0, 69]
    post_period = [70, 99]

    ci = CausalImpact(data, pre_period, post_period)
    assert int(ci.summary_data['average']['actual']) == 105
    assert int(ci.summary_data['average']['predicted']) == 100
    assert int(ci.summary_data['average']['predicted_lower']) == 99
    assert int(ci.summary_data['average']['predicted_upper']) == 100
    assert int(ci.summary_data['average']['abs_effect']) == 5
    assert int(ci.summary_data['average']['abs_effect_lower']) == 4
    assert int(ci.summary_data['average']['abs_effect_upper']) == 5


def test_default_model_sparse_linear_regression_arma_data():
    data = pd.read_csv('tests/fixtures/arma_sparse_reg.csv')
    data.iloc[70:, 0] += 5

    pre_period = [0, 69]
    post_period = [70, 99]

    ci = CausalImpact(data, pre_period, post_period)
    samples = ci.model_samples

    # Weights are computed as per original TFP source code:
    # https://github.com/tensorflow/probability/blob/v0.12.1/tensorflow_probability/python/sts/regression.py#L489-L494 # noqa: E501
    global_scale = (
        samples['SparseLinearRegression/_global_scale_noncentered'] *
        tf.sqrt(samples['SparseLinearRegression/_global_scale_variance']) * 0.1
    )
    local_scales = (
        samples['SparseLinearRegression/_local_scales_noncentered'] *
        tf.sqrt(samples['SparseLinearRegression/_local_scale_variances'])
    )
    weights = (
        samples['SparseLinearRegression/_weights_noncentered'] * local_scales *
        global_scale[..., tf.newaxis]
    )
    assert tf.abs(tf.reduce_mean(weights, axis=0).numpy()[1]) < 0.05


def test_data_no_freq():
    data = pd.read_csv('tests/fixtures/btc.csv', parse_dates=True, index_col='Date')
    training_start = "2020-12-01"
    training_end = "2021-02-05"
    treatment_start = "2021-02-08"
    treatment_end = "2021-02-09"
    pre_period = [training_start, training_end]
    post_period = [treatment_start, treatment_end]

    ci = CausalImpact(data, pre_period, post_period)

    freq_data = tfp.sts.regularize_series(data)
    pre_data = freq_data.loc[pre_period[0]: pre_period[1]]
    post_data = freq_data.loc[post_period[0]: post_period[1]]

    assert len(ci.inferences) == len(pre_data) + len(post_data)


def test_post_data_nan_values_removed():
    """
    Related to issue #51 where some points in post_data can end up having `NaN` values
    """
    data = pd.DataFrame(np.random.rand(100, 2))
    data.set_index(pd.date_range('20200101', periods=len(data)), inplace=True)
    data.iloc[:, 0].loc['20200302':] += 1
    _mask_ = data.index.isin(['20200303', '20200304', '20200404']) == False  # noqa: E712
    data = data[_mask_]
    expected_mask = np.array([True] * (39))  # 39 post data points
    expected_mask[1] = False  # '20200303'
    expected_mask[2] = False  # '20200304'
    expected_mask[33] = False  # '20200404'
    pre_period = ['20200101', '20200301']
    post_period = ['20200302', '20200409']
    ci = CausalImpact(data, pre_period, post_period)

    np.testing.assert_equal(ci._mask, expected_mask)
    assert '20200303' not in ci.inferences
    assert '20200304' not in ci.inferences
    assert '20200404' not in ci.inferences
    inferences = ci.inferences.loc[post_period[0]: post_period[1]]
    assert inferences.post_cum_y.isna().sum() == 0

    assert ci.summary_data['average']['actual'] > 0
    assert ci.summary_data['cumulative']['actual'] > 0
    assert ci.summary_data['average']['predicted'] > 0
    assert ci.summary_data['cumulative']['predicted'] > 0
    assert ci.summary_data['average']['abs_effect'] > 0
    assert ci.summary_data['cumulative']['abs_effect'] > 0


def test_custom_model_post_data_with_index_freq_holes():
    data = pd.DataFrame(np.random.rand(100, 2)).astype(np.float32)
    data.set_index(pd.date_range('20200101', periods=len(data)), inplace=True)
    pre_period = ['20200101', '20200301']
    post_period = ['20200302', '20200409']
    data = tfp.sts.regularize_series(data)
    pre_data = data.loc[pre_period[0]: pre_period[1]]
    data.iloc[:, 0].loc['20200302':] += 1
    _mask_ = data.index.isin(['20200303', '20200304', '20200404']) == False  # noqa: E712
    data = data[_mask_]
    # Apply regularize again to force design matrix to have proper shape
    reg_data = tfp.sts.regularize_series(data)
    expected_mask = np.array([True] * (39))  # 39 post data points
    expected_mask[1] = False  # '20200303'
    expected_mask[2] = False  # '20200304'
    expected_mask[33] = False  # '20200404'
    observed_time_series = pre_data.iloc[:, 0].astype(np.float32)
    level = tfp.sts.LocalLevel(observed_time_series=observed_time_series)
    design_matrix_data = reg_data.iloc[:, 1:].fillna(0).values.reshape(
        -1, data.shape[1] - 1)
    linear = tfp.sts.LinearRegression(design_matrix=design_matrix_data)
    model = tfp.sts.Sum([level, linear], observed_time_series=observed_time_series)
    ci = CausalImpact(data, pre_period, post_period, model=model,
                      model_args={'standardize': False})

    np.testing.assert_equal(ci._mask, expected_mask)
    assert '20200303' not in ci.inferences
    assert '20200304' not in ci.inferences
    assert '20200404' not in ci.inferences
    inferences = ci.inferences.loc[post_period[0]: post_period[1]]
    assert inferences.post_cum_y.isna().sum() == 0

    assert ci.summary_data['average']['actual'] > 0
    assert ci.summary_data['cumulative']['actual'] > 0
    assert ci.summary_data['average']['predicted'] > 0
    assert ci.summary_data['cumulative']['predicted'] > 0
    assert ci.summary_data['average']['abs_effect'] > 0
    assert ci.summary_data['cumulative']['abs_effect'] > 0



# Copyright WillianFuks
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""
Tests for module plot.py. Module matplotlib is not required as it's mocked accordingly.
"""


import mock
import numpy as np
import pandas as pd
import pytest
from numpy.testing import assert_array_equal

import causalimpact.plot as plotter


@pytest.fixture
def inferences(rand_data):
    df = pd.DataFrame(np.random.rand(len(rand_data), 9))
    df.columns = [
        'complete_preds_means',
        'complete_preds_lower',
        'complete_preds_upper',
        'point_effects_means',
        'point_effects_lower',
        'point_effects_upper',
        'post_cum_effects_means',
        'post_cum_effects_lower',
        'post_cum_effects_upper'
    ]
    return df


def test_build_data():
    pre_data = pd.DataFrame([0, 1, np.nan])
    # `post_data` is assumed to not have `NaN` values already
    post_data = pd.DataFrame([3, 4], index=[3, 4])
    inferences = pd.DataFrame([0, 1, 2, 3, 4])

    pre_data, post_data, inferences = plotter.build_data(pre_data, post_data, inferences)

    expected_pre_data = pd.DataFrame([0, 1]).astype(np.float64)
    pd.testing.assert_frame_equal(pre_data, expected_pre_data)

    expected_post_data = pd.DataFrame([3, 4], index=[3, 4]).astype(np.float64)
    pd.testing.assert_frame_equal(post_data, expected_post_data)

    expected_inferences = pd.DataFrame([0, 1, 3, 4],
                                       index=[0, 1, 3, 4]).astype(np.float64)
    pd.testing.assert_frame_equal(inferences, expected_inferences)


def test_plot_original_panel(rand_data, pre_int_period, post_int_period, inferences,
                             monkeypatch):
    plot_mock = mock.Mock()
    pre_data = rand_data.loc[pre_int_period[0]: pre_int_period[1]]
    post_data = rand_data.loc[post_int_period[0]: post_int_period[1]]
    pre_post_index = pre_data.index.union(post_data.index)
    monkeypatch.setattr('causalimpact.plot.get_plotter', plot_mock)
    plotter.plot(inferences, pre_data, post_data, panels=['original'])
    plot_mock.assert_called_once()
    plot_mock.return_value.figure.assert_called_with(figsize=(10, 7))
    plot_mock.return_value.subplot.assert_any_call(1, 1, 1)
    ax_mock = plot_mock.return_value.subplot.return_value
    ax_args = ax_mock.plot.call_args_list

    assert_array_equal(pre_post_index, ax_args[0][0][0])
    assert_array_equal(
        pd.concat([pre_data.iloc[:, 0], post_data.iloc[:, 0]]),
        ax_args[0][0][1]
    )
    assert ax_args[0][0][2] == 'k'
    assert ax_args[0][1] == {'label': 'y'}
    assert_array_equal(pre_post_index[1:], ax_args[1][0][0])
    assert_array_equal(inferences['complete_preds_means'].iloc[1:], ax_args[1][0][1])
    assert ax_args[1][1] == {'color': 'orangered', 'ls': 'dashed', 'label': 'Predicted'}

    ax_mock.axvline.assert_called_with(pre_int_period[1], c='gray', linestyle='--')

    ax_args = ax_mock.fill_between.call_args_list[0]
    assert_array_equal(ax_args[0][0], pre_post_index[1:])
    assert_array_equal(ax_args[0][1], inferences['complete_preds_lower'].iloc[1:])
    assert_array_equal(ax_args[0][2], inferences['complete_preds_upper'].iloc[1:])
    assert ax_args[1] == {'color': (1.0, 0.4981, 0.0549), 'alpha': 0.4}

    ax_mock.grid.assert_called_with(True, color='gainsboro')
    ax_mock.legend.assert_called()
    plot_mock.return_value.show.assert_called_once()


def test_plot_original_panel_gap_data(rand_data, pre_int_gap_period, post_int_gap_period,
                                      inferences, monkeypatch):
    plot_mock = mock.Mock()
    pre_data = rand_data.loc[pre_int_gap_period[0]: pre_int_gap_period[1]]
    post_data = rand_data.loc[post_int_gap_period[0]: post_int_gap_period[1]]

    pre_data = pre_data.set_index(pd.RangeIndex(start=0, stop=len(pre_data)))
    post_data = post_data.set_index(pd.RangeIndex(start=len(pre_data),
                                    stop=len(pre_data) + len(post_data)))

    pre_post_index = pre_data.index.union(post_data.index)
    monkeypatch.setattr('causalimpact.plot.get_plotter', plot_mock)
    plotter.plot(inferences, pre_data, post_data, panels=['original'])
    plot_mock.assert_called_once()
    plot_mock.return_value.figure.assert_called_with(figsize=(10, 7))
    plot_mock.return_value.subplot.assert_any_call(1, 1, 1)
    ax_mock = plot_mock.return_value.subplot.return_value
    ax_args = ax_mock.plot.call_args_list

    assert_array_equal(pre_post_index, ax_args[0][0][0])
    assert_array_equal(
        pd.concat([pre_data.iloc[:, 0], post_data.iloc[:, 0]]),
        ax_args[0][0][1]
    )
    assert ax_args[0][0][2] == 'k'
    assert ax_args[0][1] == {'label': 'y'}
    assert_array_equal(pre_post_index[1:], ax_args[1][0][0])
    assert_array_equal(inferences['complete_preds_means'].iloc[1:], ax_args[1][0][1])
    assert ax_args[1][1] == {'color': 'orangered', 'ls': 'dashed', 'label': 'Predicted'}

    ax_mock.axvline.assert_called_with(pre_data.index[-1], c='gray', linestyle='--')

    ax_args = ax_mock.fill_between.call_args_list[0]
    assert_array_equal(ax_args[0][0], pre_post_index[1:])
    assert_array_equal(ax_args[0][1], inferences['complete_preds_lower'].iloc[1:])
    assert_array_equal(ax_args[0][2], inferences['complete_preds_upper'].iloc[1:])
    assert ax_args[1] == {'color': (1.0, 0.4981, 0.0549), 'alpha': 0.4}

    ax_mock.grid.assert_called_with(True, color='gainsboro')
    ax_mock.legend.assert_called()
    plot_mock.return_value.show.assert_called_once()


def test_plot_original_panel_date_index(date_rand_data, pre_str_period, post_str_period,
                                        inferences, monkeypatch):
    plot_mock = mock.Mock()
    pre_data = date_rand_data.loc[pre_str_period[0]: pre_str_period[1]]
    post_data = date_rand_data.loc[post_str_period[0]: post_str_period[1]]
    pre_post_index = pre_data.index.union(post_data.index)
    monkeypatch.setattr('causalimpact.plot.get_plotter', plot_mock)
    inferences = inferences.set_index(pre_post_index)
    plotter.plot(inferences, pre_data, post_data, panels=['original'])
    plot_mock.assert_called_once()
    plot_mock.return_value.figure.assert_called_with(figsize=(10, 7))
    plot_mock.return_value.subplot.assert_any_call(1, 1, 1)
    ax_mock = plot_mock.return_value.subplot.return_value
    ax_args = ax_mock.plot.call_args_list

    assert_array_equal(pre_post_index, ax_args[0][0][0])
    assert_array_equal(
        pd.concat([pre_data.iloc[:, 0], post_data.iloc[:, 0]]),
        ax_args[0][0][1]
    )
    assert ax_args[0][0][2] == 'k'
    assert ax_args[0][1] == {'label': 'y'}
    assert_array_equal(pre_post_index[1:], ax_args[1][0][0])
    assert_array_equal(inferences['complete_preds_means'].iloc[1:], ax_args[1][0][1])
    assert ax_args[1][1] == {'color': 'orangered', 'ls': 'dashed', 'label': 'Predicted'}

    idx_value = pre_post_index.get_loc(pre_str_period[1])
    ax_mock.axvline.assert_called_with(pre_post_index[idx_value], c='gray',
                                       linestyle='--')

    ax_args = ax_mock.fill_between.call_args_list[0]
    assert_array_equal(ax_args[0][0], pre_post_index[1:])
    assert_array_equal(ax_args[0][1], inferences['complete_preds_lower'].iloc[1:])
    assert_array_equal(ax_args[0][2], inferences['complete_preds_upper'].iloc[1:])
    assert ax_args[1] == {'color': (1.0, 0.4981, 0.0549), 'alpha': 0.4}

    ax_mock.grid.assert_called_with(True, color='gainsboro')
    ax_mock.legend.assert_called()
    plot_mock.return_value.show.assert_called_once()


def test_plot_original_panel_gap_date_index(date_rand_data, pre_str_gap_period,
                                            post_str_gap_period, inferences,
                                            monkeypatch):
    plot_mock = mock.Mock()
    pre_data = date_rand_data.loc[pre_str_gap_period[0]: pre_str_gap_period[1]]
    post_data = date_rand_data.loc[post_str_gap_period[0]: post_str_gap_period[1]]
    pre_post_index = pre_data.index.union(post_data.index)
    inferences = inferences.set_index(
        pd.date_range(start=pre_post_index[0], periods=len(inferences))
    )
    monkeypatch.setattr('causalimpact.plot.get_plotter', plot_mock)
    plotter.plot(inferences, pre_data, post_data, panels=['original'])
    plot_mock.assert_called_once()
    plot_mock.return_value.figure.assert_called_with(figsize=(10, 7))
    plot_mock.return_value.subplot.assert_any_call(1, 1, 1)
    ax_mock = plot_mock.return_value.subplot.return_value
    ax_args = ax_mock.plot.call_args_list

    assert_array_equal(pre_post_index, ax_args[0][0][0])
    assert_array_equal(
        pd.concat([pre_data.iloc[:, 0], post_data.iloc[:, 0]]),
        ax_args[0][0][1]
    )
    assert ax_args[0][0][2] == 'k'
    assert ax_args[0][1] == {'label': 'y'}
    assert_array_equal(pre_post_index[1:], ax_args[1][0][0])
    assert_array_equal(inferences['complete_preds_means'].iloc[1:], ax_args[1][0][1])
    assert ax_args[1][1] == {'color': 'orangered', 'ls': 'dashed', 'label': 'Predicted'}

    idx_value = pre_post_index.get_loc(pre_str_gap_period[1])
    ax_mock.axvline.assert_called_with(pre_post_index[idx_value], c='gray',
                                       linestyle='--')

    ax_args = ax_mock.fill_between.call_args_list[0]
    assert_array_equal(ax_args[0][0], pre_post_index[1:])
    assert_array_equal(ax_args[0][1], inferences['complete_preds_lower'].iloc[1:])
    assert_array_equal(ax_args[0][2], inferences['complete_preds_upper'].iloc[1:])
    assert ax_args[1] == {'color': (1.0, 0.4981, 0.0549), 'alpha': 0.4}

    ax_mock.grid.assert_called_with(True, color='gainsboro')
    ax_mock.legend.assert_called()
    plot_mock.return_value.show.assert_called_once()


def test_plot_original_panel_date_index_no_freq(date_rand_data, pre_str_period,
                                                post_str_period, inferences,
                                                monkeypatch):
    dd = date_rand_data.copy()
    dd.drop(dd.index[10:20], inplace=True)
    plot_mock = mock.Mock()
    pre_data = dd.loc[pre_str_period[0]: pre_str_period[1]]
    post_data = dd.loc[post_str_period[0]: post_str_period[1]]
    pre_post_index = pre_data.index.union(post_data.index)
    inferences = inferences.set_index(
        pd.date_range(start=pre_post_index[0], periods=len(inferences))
    )
    monkeypatch.setattr('causalimpact.plot.get_plotter', plot_mock)
    plotter.plot(inferences, pre_data, post_data, panels=['original'])
    plot_mock.assert_called_once()
    plot_mock.return_value.figure.assert_called_with(figsize=(10, 7))
    plot_mock.return_value.subplot.assert_any_call(1, 1, 1)
    ax_mock = plot_mock.return_value.subplot.return_value
    ax_args = ax_mock.plot.call_args_list

    assert_array_equal(pre_post_index, ax_args[0][0][0])
    assert_array_equal(
        pd.concat([pre_data.iloc[:, 0], post_data.iloc[:, 0]]),
        ax_args[0][0][1]
    )
    assert ax_args[0][0][2] == 'k'
    assert ax_args[0][1] == {'label': 'y'}
    assert_array_equal(pre_post_index[1:], ax_args[1][0][0])
    assert_array_equal(inferences['complete_preds_means'].iloc[1:], ax_args[1][0][1])
    assert ax_args[1][1] == {'color': 'orangered', 'ls': 'dashed', 'label': 'Predicted'}

    idx_value = pre_post_index.get_loc(pre_str_period[1])
    ax_mock.axvline.assert_called_with(pre_post_index[idx_value], c='gray',
                                       linestyle='--')

    ax_args = ax_mock.fill_between.call_args_list[0]
    assert_array_equal(ax_args[0][0], pre_post_index[1:])
    assert_array_equal(ax_args[0][1], inferences['complete_preds_lower'].iloc[1:])
    assert_array_equal(ax_args[0][2], inferences['complete_preds_upper'].iloc[1:])
    assert ax_args[1] == {'color': (1.0, 0.4981, 0.0549), 'alpha': 0.4}

    ax_mock.grid.assert_called_with(True, color='gainsboro')
    ax_mock.legend.assert_called()
    plot_mock.return_value.show.assert_called_once()


def test_plot_pointwise_panel(rand_data, pre_int_period, post_int_period, inferences,
                              monkeypatch):
    plot_mock = mock.Mock()
    pre_data = rand_data.loc[pre_int_period[0]: pre_int_period[1]]
    post_data = rand_data.loc[post_int_period[0]: post_int_period[1]]
    pre_post_index = pre_data.index.union(post_data.index)
    monkeypatch.setattr('causalimpact.plot.get_plotter', plot_mock)
    plotter.plot(inferences, pre_data, post_data, panels=['pointwise'])
    plot_mock.assert_called_once()
    plot_mock.return_value.figure.assert_called_with(figsize=(10, 7))
    plot_mock.return_value.subplot.assert_any_call(1, 1, 1)
    ax_mock = plot_mock.return_value.subplot.return_value
    ax_args = ax_mock.plot.call_args_list

    assert_array_equal(pre_post_index[1:], ax_args[0][0][0])
    assert_array_equal(
        inferences['point_effects_means'][1:],
        ax_args[0][0][1]
    )
    assert ax_args[0][1] == {'label': 'Point Effects', 'ls': 'dashed',
                             'color': 'orangered'}
    ax_mock.axvline.assert_called_with(pre_int_period[1], c='gray', linestyle='--')

    ax_args = ax_mock.fill_between.call_args_list[0]
    assert_array_equal(ax_args[0][0], pre_post_index[1:])
    assert_array_equal(ax_args[0][1], inferences['point_effects_lower'].iloc[1:])
    assert_array_equal(ax_args[0][2], inferences['point_effects_upper'].iloc[1:])
    assert ax_args[1] == {'color': (1.0, 0.4981, 0.0549), 'alpha': 0.4}

    ax_mock.grid.assert_called_with(True, color='gainsboro')
    ax_mock.legend.assert_called()
    plot_mock.return_value.show.assert_called_once()


def test_plot_pointwise_panel_gap_data(rand_data, pre_int_gap_period,
                                       post_int_gap_period, inferences,
                                       monkeypatch):
    plot_mock = mock.Mock()
    pre_data = rand_data.loc[pre_int_gap_period[0]: pre_int_gap_period[1]]
    post_data = rand_data.loc[post_int_gap_period[0]: post_int_gap_period[1]]

    pre_data = pre_data.set_index(pd.RangeIndex(start=0, stop=len(pre_data)))
    post_data = post_data.set_index(pd.RangeIndex(start=len(pre_data),
                                    stop=len(pre_data) + len(post_data)))

    pre_post_index = pre_data.index.union(post_data.index)
    monkeypatch.setattr('causalimpact.plot.get_plotter', plot_mock)
    plotter.plot(inferences, pre_data, post_data, panels=['pointwise'])
    plot_mock.assert_called_once()
    plot_mock.return_value.figure.assert_called_with(figsize=(10, 7))
    plot_mock.return_value.subplot.assert_any_call(1, 1, 1)
    ax_mock = plot_mock.return_value.subplot.return_value
    ax_args = ax_mock.plot.call_args_list

    assert_array_equal(pre_post_index[1:], ax_args[0][0][0])
    assert_array_equal(
        inferences['point_effects_means'][1:],
        ax_args[0][0][1]
    )
    assert ax_args[0][1] == {'label': 'Point Effects', 'ls': 'dashed',
                             'color': 'orangered'}
    ax_mock.axvline.assert_called_with(pre_data.index[-1], c='gray', linestyle='--')

    ax_args = ax_mock.fill_between.call_args_list[0]
    assert_array_equal(ax_args[0][0], pre_post_index[1:])
    assert_array_equal(ax_args[0][1], inferences['point_effects_lower'].iloc[1:])
    assert_array_equal(ax_args[0][2], inferences['point_effects_upper'].iloc[1:])
    assert ax_args[1] == {'color': (1.0, 0.4981, 0.0549), 'alpha': 0.4}

    ax_mock.grid.assert_called_with(True, color='gainsboro')
    ax_mock.legend.assert_called()
    plot_mock.return_value.show.assert_called_once()


def test_plot_pointwise_panel_date_index(date_rand_data, pre_str_period, post_str_period,
                                         inferences, monkeypatch):
    plot_mock = mock.Mock()
    pre_data = date_rand_data.loc[pre_str_period[0]: pre_str_period[1]]
    post_data = date_rand_data.loc[post_str_period[0]: post_str_period[1]]
    pre_post_index = pre_data.index.union(post_data.index)
    inferences = inferences.set_index(pre_post_index)
    monkeypatch.setattr('causalimpact.plot.get_plotter', plot_mock)
    plotter.plot(inferences, pre_data, post_data, panels=['pointwise'])
    plot_mock.assert_called_once()
    plot_mock.return_value.figure.assert_called_with(figsize=(10, 7))
    plot_mock.return_value.subplot.assert_any_call(1, 1, 1)
    ax_mock = plot_mock.return_value.subplot.return_value
    ax_args = ax_mock.plot.call_args_list

    assert_array_equal(pre_post_index[1:], ax_args[0][0][0])
    assert_array_equal(
        inferences['point_effects_means'][1:],
        ax_args[0][0][1]
    )
    assert ax_args[0][1] == {'label': 'Point Effects', 'ls': 'dashed',
                             'color': 'orangered'}
    idx_value = pre_post_index.get_loc(pre_str_period[1])
    ax_mock.axvline.assert_called_with(pre_post_index[idx_value], c='gray',
                                       linestyle='--')

    ax_args = ax_mock.fill_between.call_args_list[0]
    assert_array_equal(ax_args[0][0], pre_post_index[1:])
    assert_array_equal(ax_args[0][1], inferences['point_effects_lower'].iloc[1:])
    assert_array_equal(ax_args[0][2], inferences['point_effects_upper'].iloc[1:])
    assert ax_args[1] == {'color': (1.0, 0.4981, 0.0549), 'alpha': 0.4}

    ax_mock.grid.assert_called_with(True, color='gainsboro')
    ax_mock.legend.assert_called()
    plot_mock.return_value.show.assert_called_once()


def test_plot_pointwise_panel_gap_date_index(date_rand_data, pre_str_gap_period,
                                             post_str_gap_period, inferences,
                                             monkeypatch):
    plot_mock = mock.Mock()
    pre_data = date_rand_data.loc[pre_str_gap_period[0]: pre_str_gap_period[1]]
    post_data = date_rand_data.loc[post_str_gap_period[0]: post_str_gap_period[1]]
    pre_post_index = pre_data.index.union(post_data.index)
    inferences = inferences.set_index(
        pd.date_range(start=pre_post_index[0], periods=len(inferences))
    )
    monkeypatch.setattr('causalimpact.plot.get_plotter', plot_mock)
    plotter.plot(inferences, pre_data, post_data, panels=['pointwise'])
    plot_mock.assert_called_once()
    plot_mock.return_value.figure.assert_called_with(figsize=(10, 7))
    plot_mock.return_value.subplot.assert_any_call(1, 1, 1)
    ax_mock = plot_mock.return_value.subplot.return_value
    ax_args = ax_mock.plot.call_args_list

    assert_array_equal(pre_post_index[1:], ax_args[0][0][0])
    assert_array_equal(
        inferences['point_effects_means'][1:],
        ax_args[0][0][1]
    )
    assert ax_args[0][1] == {'label': 'Point Effects', 'ls': 'dashed',
                             'color': 'orangered'}
    idx_value = pre_post_index.get_loc(pre_str_gap_period[1])
    ax_mock.axvline.assert_called_with(pre_post_index[idx_value], c='gray',
                                       linestyle='--')

    ax_args = ax_mock.fill_between.call_args_list[0]
    assert_array_equal(ax_args[0][0], pre_post_index[1:])
    assert_array_equal(ax_args[0][1], inferences['point_effects_lower'].iloc[1:])
    assert_array_equal(ax_args[0][2], inferences['point_effects_upper'].iloc[1:])
    assert ax_args[1] == {'color': (1.0, 0.4981, 0.0549), 'alpha': 0.4}

    ax_mock.grid.assert_called_with(True, color='gainsboro')
    ax_mock.legend.assert_called()
    plot_mock.return_value.show.assert_called_once()


def test_plot_pointwise_panel_date_index_no_freq(date_rand_data, pre_str_period,
                                                 post_str_period, inferences,
                                                 monkeypatch):
    dd = date_rand_data.copy()
    dd.drop(dd.index[10:20], inplace=True)
    plot_mock = mock.Mock()
    pre_data = dd.loc[pre_str_period[0]: pre_str_period[1]]
    post_data = dd.loc[post_str_period[0]: post_str_period[1]]
    pre_post_index = pre_data.index.union(post_data.index)
    inferences = inferences.set_index(
        pd.date_range(start=pre_post_index[0], periods=len(inferences))
    )
    monkeypatch.setattr('causalimpact.plot.get_plotter', plot_mock)
    plotter.plot(inferences, pre_data, post_data, panels=['pointwise'])
    plot_mock.assert_called_once()
    plot_mock.return_value.figure.assert_called_with(figsize=(10, 7))
    plot_mock.return_value.subplot.assert_any_call(1, 1, 1)
    ax_mock = plot_mock.return_value.subplot.return_value
    ax_args = ax_mock.plot.call_args_list

    assert_array_equal(pre_post_index[1:], ax_args[0][0][0])
    assert_array_equal(
        inferences['point_effects_means'][1:],
        ax_args[0][0][1]
    )
    assert ax_args[0][1] == {'label': 'Point Effects', 'ls': 'dashed',
                             'color': 'orangered'}
    idx_value = pre_post_index.get_loc(pre_str_period[1])
    ax_mock.axvline.assert_called_with(pre_post_index[idx_value], c='gray',
                                       linestyle='--')

    ax_args = ax_mock.fill_between.call_args_list[0]
    assert_array_equal(ax_args[0][0], pre_post_index[1:])
    assert_array_equal(ax_args[0][1], inferences['point_effects_lower'].iloc[1:])
    assert_array_equal(ax_args[0][2], inferences['point_effects_upper'].iloc[1:])
    assert ax_args[1] == {'color': (1.0, 0.4981, 0.0549), 'alpha': 0.4}

    ax_mock.grid.assert_called_with(True, color='gainsboro')
    ax_mock.legend.assert_called()
    plot_mock.return_value.show.assert_called_once()


def test_plot_cumulative_panel(rand_data, pre_int_period, post_int_period, inferences,
                               monkeypatch):
    plot_mock = mock.Mock()
    pre_data = rand_data.loc[pre_int_period[0]: pre_int_period[1]]
    post_data = rand_data.loc[post_int_period[0]: post_int_period[1]]
    pre_post_index = pre_data.index.union(post_data.index)
    monkeypatch.setattr('causalimpact.plot.get_plotter', plot_mock)
    plotter.plot(inferences, pre_data, post_data, panels=['cumulative'])
    plot_mock.assert_called_once()
    plot_mock.return_value.figure.assert_called_with(figsize=(10, 7))
    plot_mock.return_value.subplot.assert_any_call(1, 1, 1)
    ax_mock = plot_mock.return_value.subplot.return_value
    ax_args = ax_mock.plot.call_args_list

    assert_array_equal(pre_post_index[1:], ax_args[0][0][0])
    assert_array_equal(
        inferences['post_cum_effects_means'][1:],
        ax_args[0][0][1]
    )
    assert ax_args[0][1] == {'label': 'Cumulative Effect', 'ls': 'dashed',
                             'color': 'orangered'}
    ax_mock.axvline.assert_called_with(pre_int_period[1], c='gray', linestyle='--')

    ax_args = ax_mock.fill_between.call_args_list[0]
    assert_array_equal(ax_args[0][0], pre_post_index[1:])
    assert_array_equal(ax_args[0][1], inferences['post_cum_effects_lower'].iloc[1:])
    assert_array_equal(ax_args[0][2], inferences['post_cum_effects_upper'].iloc[1:])
    assert ax_args[1] == {'color': (1.0, 0.4981, 0.0549), 'alpha': 0.4}

    ax_mock.grid.assert_called_with(True, color='gainsboro')
    ax_mock.legend.assert_called()
    plot_mock.return_value.show.assert_called_once()


def test_plot_cumulative_panel_gap_data(rand_data, pre_int_gap_period,
                                        post_int_gap_period, inferences, monkeypatch):
    plot_mock = mock.Mock()
    pre_data = rand_data.loc[pre_int_gap_period[0]: pre_int_gap_period[1]]
    post_data = rand_data.loc[post_int_gap_period[0]: post_int_gap_period[1]]

    pre_data = pre_data.set_index(pd.RangeIndex(start=0, stop=len(pre_data)))
    post_data = post_data.set_index(pd.RangeIndex(start=len(pre_data),
                                    stop=len(pre_data) + len(post_data)))

    pre_post_index = pre_data.index.union(post_data.index)
    monkeypatch.setattr('causalimpact.plot.get_plotter', plot_mock)
    plotter.plot(inferences, pre_data, post_data, panels=['cumulative'])
    plot_mock.assert_called_once()
    plot_mock.return_value.figure.assert_called_with(figsize=(10, 7))
    plot_mock.return_value.subplot.assert_any_call(1, 1, 1)
    ax_mock = plot_mock.return_value.subplot.return_value
    ax_args = ax_mock.plot.call_args_list

    assert_array_equal(pre_post_index[1:], ax_args[0][0][0])
    assert_array_equal(
        inferences['post_cum_effects_means'][1:],
        ax_args[0][0][1]
    )
    assert ax_args[0][1] == {'label': 'Cumulative Effect', 'ls': 'dashed',
                             'color': 'orangered'}
    ax_mock.axvline.assert_called_with(pre_data.index[-1], c='gray', linestyle='--')

    ax_args = ax_mock.fill_between.call_args_list[0]
    assert_array_equal(ax_args[0][0], pre_post_index[1:])
    assert_array_equal(ax_args[0][1], inferences['post_cum_effects_lower'].iloc[1:])
    assert_array_equal(ax_args[0][2], inferences['post_cum_effects_upper'].iloc[1:])
    assert ax_args[1] == {'color': (1.0, 0.4981, 0.0549), 'alpha': 0.4}

    ax_mock.grid.assert_called_with(True, color='gainsboro')
    ax_mock.legend.assert_called()
    plot_mock.return_value.show.assert_called_once()


def test_plot_cumulative_panel_date_index(date_rand_data, pre_str_period,
                                          post_str_period, inferences, monkeypatch):
    plot_mock = mock.Mock()
    pre_data = date_rand_data.loc[pre_str_period[0]: pre_str_period[1]]
    post_data = date_rand_data.loc[post_str_period[0]: post_str_period[1]]
    pre_post_index = pre_data.index.union(post_data.index)
    inferences = inferences.set_index(pre_post_index)
    monkeypatch.setattr('causalimpact.plot.get_plotter', plot_mock)
    plotter.plot(inferences, pre_data, post_data, panels=['cumulative'])
    plot_mock.assert_called_once()
    plot_mock.return_value.figure.assert_called_with(figsize=(10, 7))
    plot_mock.return_value.subplot.assert_any_call(1, 1, 1)
    ax_mock = plot_mock.return_value.subplot.return_value
    ax_args = ax_mock.plot.call_args_list

    assert_array_equal(pre_post_index[1:], ax_args[0][0][0])
    assert_array_equal(
        inferences['post_cum_effects_means'][1:],
        ax_args[0][0][1]
    )
    assert ax_args[0][1] == {'label': 'Cumulative Effect', 'ls': 'dashed',
                             'color': 'orangered'}
    idx_value = pre_post_index.get_loc(pre_str_period[1])
    ax_mock.axvline.assert_called_with(pre_post_index[idx_value], c='gray',
                                       linestyle='--')

    ax_args = ax_mock.fill_between.call_args_list[0]
    assert_array_equal(ax_args[0][0], pre_post_index[1:])
    assert_array_equal(ax_args[0][1], inferences['post_cum_effects_lower'].iloc[1:])
    assert_array_equal(ax_args[0][2], inferences['post_cum_effects_upper'].iloc[1:])
    assert ax_args[1] == {'color': (1.0, 0.4981, 0.0549), 'alpha': 0.4}

    ax_mock.grid.assert_called_with(True, color='gainsboro')
    ax_mock.legend.assert_called()
    plot_mock.return_value.show.assert_called_once()


def test_plot_cumulative_panel_gap_date_index(date_rand_data, pre_str_gap_period,
                                              post_str_gap_period, inferences,
                                              monkeypatch):
    plot_mock = mock.Mock()
    pre_data = date_rand_data.loc[pre_str_gap_period[0]: pre_str_gap_period[1]]
    post_data = date_rand_data.loc[post_str_gap_period[0]: post_str_gap_period[1]]
    pre_post_index = pre_data.index.union(post_data.index)
    inferences = inferences.set_index(
        pd.date_range(start=pre_post_index[0], periods=len(inferences))
    )
    monkeypatch.setattr('causalimpact.plot.get_plotter', plot_mock)
    plotter.plot(inferences, pre_data, post_data, panels=['cumulative'])
    plot_mock.assert_called_once()
    plot_mock.return_value.figure.assert_called_with(figsize=(10, 7))
    plot_mock.return_value.subplot.assert_any_call(1, 1, 1)
    ax_mock = plot_mock.return_value.subplot.return_value
    ax_args = ax_mock.plot.call_args_list

    assert_array_equal(pre_post_index[1:], ax_args[0][0][0])
    assert_array_equal(
        inferences['post_cum_effects_means'][1:],
        ax_args[0][0][1]
    )
    assert ax_args[0][1] == {'label': 'Cumulative Effect', 'ls': 'dashed',
                             'color': 'orangered'}
    idx_value = pre_post_index.get_loc(pre_str_gap_period[1])
    ax_mock.axvline.assert_called_with(pre_post_index[idx_value], c='gray',
                                       linestyle='--')

    ax_args = ax_mock.fill_between.call_args_list[0]
    assert_array_equal(ax_args[0][0], pre_post_index[1:])
    assert_array_equal(ax_args[0][1], inferences['post_cum_effects_lower'].iloc[1:])
    assert_array_equal(ax_args[0][2], inferences['post_cum_effects_upper'].iloc[1:])
    assert ax_args[1] == {'color': (1.0, 0.4981, 0.0549), 'alpha': 0.4}

    ax_mock.grid.assert_called_with(True, color='gainsboro')
    ax_mock.legend.assert_called()
    plot_mock.return_value.show.assert_called_once()


def test_plot_cumulative_panel_date_index_no_freq(date_rand_data, pre_str_period,
                                                  post_str_period, inferences,
                                                  monkeypatch):
    dd = date_rand_data.copy()
    dd.drop(dd.index[10:20], inplace=True)
    plot_mock = mock.Mock()
    pre_data = dd.loc[pre_str_period[0]: pre_str_period[1]]
    post_data = dd.loc[post_str_period[0]: post_str_period[1]]
    pre_post_index = pre_data.index.union(post_data.index)
    inferences = inferences.set_index(
        pd.date_range(start=pre_post_index[0], periods=len(inferences))
    )
    monkeypatch.setattr('causalimpact.plot.get_plotter', plot_mock)
    plotter.plot(inferences, pre_data, post_data, panels=['cumulative'])
    plot_mock.assert_called_once()
    plot_mock.return_value.figure.assert_called_with(figsize=(10, 7))
    plot_mock.return_value.subplot.assert_any_call(1, 1, 1)
    ax_mock = plot_mock.return_value.subplot.return_value
    ax_args = ax_mock.plot.call_args_list

    assert_array_equal(pre_post_index[1:], ax_args[0][0][0])
    assert_array_equal(
        inferences['post_cum_effects_means'][1:],
        ax_args[0][0][1]
    )
    assert ax_args[0][1] == {'label': 'Cumulative Effect', 'ls': 'dashed',
                             'color': 'orangered'}
    idx_value = pre_post_index.get_loc(pre_str_period[1])
    ax_mock.axvline.assert_called_with(pre_post_index[idx_value], c='gray',
                                       linestyle='--')

    ax_args = ax_mock.fill_between.call_args_list[0]
    assert_array_equal(ax_args[0][0], pre_post_index[1:])
    assert_array_equal(ax_args[0][1], inferences['post_cum_effects_lower'].iloc[1:])
    assert_array_equal(ax_args[0][2], inferences['post_cum_effects_upper'].iloc[1:])
    assert ax_args[1] == {'color': (1.0, 0.4981, 0.0549), 'alpha': 0.4}

    ax_mock.grid.assert_called_with(True, color='gainsboro')
    ax_mock.legend.assert_called()
    plot_mock.return_value.show.assert_called_once()


def test_plot_multi_panels(rand_data, pre_int_period, post_int_period, inferences,
                           monkeypatch):
    plot_mock = mock.Mock()
    pre_data = rand_data.loc[pre_int_period[0]: pre_int_period[1]]
    post_data = rand_data.loc[post_int_period[0]: post_int_period[1]]
    monkeypatch.setattr('causalimpact.plot.get_plotter', plot_mock)
    plotter.plot(inferences, pre_data, post_data, panels=['original', 'pointwise'])
    plot_mock.assert_called_once()
    plot_mock.return_value.figure.assert_called_with(figsize=(10, 7))
    ax_mock = plot_mock.return_value.subplot.return_value
    assert ax_mock.plot.call_count == 3
    assert ax_mock.axvline.call_count == 2
    assert ax_mock.grid.call_count == 2
    assert ax_mock.legend.call_count == 2
    assert ax_mock.fill_between.call_count == 2
    assert ax_mock.axhline.call_count == 1
    assert plot_mock.return_value.setp.call_count == 1

    plot_mock.reset_mock()
    ax_mock.reset_mock()
    plotter.plot(inferences, pre_data, post_data, panels=['original', 'cumulative'])
    plot_mock.assert_called_once()
    plot_mock.return_value.figure.assert_called_with(figsize=(10, 7))
    ax_mock = plot_mock.return_value.subplot.return_value
    assert ax_mock.plot.call_count == 3
    assert ax_mock.axvline.call_count == 2
    assert ax_mock.grid.call_count == 2
    assert ax_mock.legend.call_count == 2
    assert ax_mock.fill_between.call_count == 2
    assert ax_mock.axhline.call_count == 1
    assert plot_mock.return_value.setp.call_count == 1

    plot_mock.reset_mock()
    ax_mock.reset_mock()
    plotter.plot(inferences, pre_data, post_data, panels=['original', 'pointwise',
                 'cumulative'])
    plot_mock.assert_called_once()
    plot_mock.return_value.figure.assert_called_with(figsize=(10, 7))
    ax_mock = plot_mock.return_value.subplot.return_value
    assert ax_mock.plot.call_count == 4
    assert ax_mock.axvline.call_count == 3
    assert ax_mock.grid.call_count == 3
    assert ax_mock.legend.call_count == 3
    assert ax_mock.fill_between.call_count == 3
    assert ax_mock.axhline.call_count == 2
    assert plot_mock.return_value.setp.call_count == 2

    plot_mock.reset_mock()
    ax_mock.reset_mock()
    plotter.plot(inferences, pre_data, post_data, panels=['pointwise', 'cumulative'])
    plot_mock.assert_called_once()
    plot_mock.return_value.figure.assert_called_with(figsize=(10, 7))
    ax_mock = plot_mock.return_value.subplot.return_value
    assert ax_mock.plot.call_count == 2
    assert ax_mock.axvline.call_count == 2
    assert ax_mock.grid.call_count == 2
    assert ax_mock.legend.call_count == 2
    assert ax_mock.fill_between.call_count == 2
    assert ax_mock.axhline.call_count == 2
    assert plot_mock.return_value.setp.call_count == 1


def test_plot_raises_wrong_input_panel(rand_data, pre_int_period, post_int_period,
                                       inferences, monkeypatch):
    plot_mock = mock.Mock()
    pre_data = rand_data.loc[pre_int_period[0]: pre_int_period[1]]
    post_data = rand_data.loc[post_int_period[0]: post_int_period[1]]
    monkeypatch.setattr('causalimpact.plot.get_plotter', plot_mock)
    plotter.plot(inferences, pre_data, post_data, panels=['cumulative'])

    with pytest.raises(ValueError) as excinfo:
        plotter.plot(inferences, rand_data, rand_data, panels=['test'])
    assert str(excinfo.value) == (
        '"test" is not a valid panel. Valid panels are: '
        '"original", "pointwise", "cumulative".'
    )


def test_plot_original_panel_gap_data_show_is_false(
    rand_data, pre_int_gap_period, post_int_gap_period, inferences, monkeypatch
):
    plot_mock = mock.Mock()
    pre_data = rand_data.loc[pre_int_gap_period[0]: pre_int_gap_period[1]]
    post_data = rand_data.loc[post_int_gap_period[0]: post_int_gap_period[1]]

    pre_data = pre_data.set_index(pd.RangeIndex(start=0, stop=len(pre_data)))
    post_data = post_data.set_index(pd.RangeIndex(start=len(pre_data),
                                    stop=len(pre_data) + len(post_data)))

    pre_post_index = pre_data.index.union(post_data.index)
    monkeypatch.setattr("causalimpact.plot.get_plotter", plot_mock)
    plotter.plot(inferences, pre_data, post_data, panels=["original"], show=False)

    plot_mock.assert_called_once()
    plot_mock.return_value.figure.assert_called_with(figsize=(10, 7))
    plot_mock.return_value.subplot.assert_any_call(1, 1, 1)
    ax_mock = plot_mock.return_value.subplot.return_value
    ax_args = ax_mock.plot.call_args_list

    assert_array_equal(pre_post_index, ax_args[0][0][0])
    assert_array_equal(
        pd.concat([pre_data.iloc[:, 0], post_data.iloc[:, 0]]), ax_args[0][0][1]
    )
    assert ax_args[0][0][2] == "k"
    assert ax_args[0][1] == {"label": "y"}
    assert_array_equal(pre_post_index[1:], ax_args[1][0][0])
    assert_array_equal(inferences["complete_preds_means"].iloc[1:], ax_args[1][0][1])
    assert ax_args[1][1] == {"color": "orangered", "ls": "dashed", "label": "Predicted"}

    ax_mock.axvline.assert_called_with(pre_data.index[-1], c="gray", linestyle="--")

    ax_args = ax_mock.fill_between.call_args_list[0]
    assert_array_equal(ax_args[0][0], pre_post_index[1:])
    assert_array_equal(ax_args[0][1], inferences["complete_preds_lower"].iloc[1:])
    assert_array_equal(ax_args[0][2], inferences["complete_preds_upper"].iloc[1:])
    assert ax_args[1] == {"color": (1.0, 0.4981, 0.0549), "alpha": 0.4}

    ax_mock.grid.assert_called_with(True, color="gainsboro")
    ax_mock.legend.assert_called()
    # If `show == False` then `plt.show()` should not have been called
    plot_mock.return_value.show.assert_not_called()
